#!/bin/bash

# Script which takes backup of computers in file "computers"
# and only from paths in file "paths".

# Usage nixbackup.sh
#
# Parameters
# none
#
# Script could be run as a cronjob, example:
# 25 4 * * * /root/bin/nixbackup/nixbackup.sh > /dev/null

# ===== START OF CONFIG, NEED TO EDIT =====
# Basic info
max_full_backup=3		#how many full backups to store
days_between_full_backup=30	#number of "daily" storages between backups

# Local directories
basedir="/root/bin/nixbackup"	#base directory where the script base is
confdir=$basedir		#configuration files
storagedevice="/dev/md1"	#device of the huge disk
storagemount="/raid/md1"	#mountpoint of the huge disk
storagedir="/raid/md1/backup"	#path on the huge disk to store backup

# Change to 1 when configured
configured=0			#set to 1 when configured

# ===== END USER EDITING, NO EDITING BELOW THIS LINE! =====

# ===== ChangeLog =====
# v1.2.0 released - 11th Mars 2008
# * Added a check for the storagemount before starting backup. This will
#   prevent the backup script for filling up the harddrive if the backup
#   device is not present.
# * The script will also try to mount the storage device on the storage mount
#   point. New configuration value added; storagedevice.
# * General cleanup in output, less verbose.
#
# v1.19 released - 13th September 2006
# * Fixed a bug when printing how large the backup volume is
#
# v1.18 released - 12th July 2006
# * Fixed depricated switch to sort command
#
# v1.17 released - 08th July 2006
# * Fixed depricated switch to tail command
# * Changed maintainer email
#
# v1.16 released - 26th May 2005
# * Fixed a bug when deleting tmp files if the script fails
# * Enhanced code readability
#
# v1.15 released - 11th May 2005
# * changed tar cl to tar c --one-file-system
# * whitespace cleanups
# * added new config, storagemount
#
# v1.14 released - 26th September 2004
# * bugfix with detection of full backup, daily now works
# * bugfix with daily backups, --newer changed to --newer-mtime
# * bugfix in backupline, changed $lastcmd to $lastday
# * code cleanup
# * added deletion of old daily backups
# * added info of size of each backup
#
# v1.13 released - 26th September 2004
# * added a ChangeLog
# * minor cosmetic changes
# * changed a while loop to a for loop
# * bug in detection of full backup
#
# ===== END ChangeLog =====

# ===== START OF SCRIPT =====
set -e
version="1.2.0"
verdate="2008-03-11"

echo
echo "Remote directory and SQL backup script"
echo "Version: $version, last updated $verdate"
echo "Maintained by Hans-Christian Egtvedt <hc@egtvedt.no>"
echo
echo "Run started at `date`"

if [ "$configured" -eq 0 ]; then
	echo
	echo "Script not configured, exiting..."
	echo
	echo "Run ended at `date`"
	echo
	exit 1
fi

# Check if the storagemount is mounted
STORAGEMOUNT_OK=`cat /proc/mounts | grep "${storagedevice} \/" | cut -d ' ' -f2`

if [ "${STORAGEMOUNT_OK}" != "${storagemount}" ]; then
	echo
	echo "Storage mount point is not mounted, trying to mount..."

	if ! mount ${storagedevice} ${storagemount}; then
		echo
		echo "Failed to mount '${storagedevice}' on '${storagemount}', exiting..."
		exit 1
	else
		echo " * Mounted '${storagedevice}' on '${storagemount}'"
	fi
fi

# The computers we want to back up
if [ -r $confdir/computers ]; then
	computers=`cat $confdir/computers`
else
	echo
	echo "Computers file does not exist, exiting..."
	echo
	exit 1
fi


# Which directories to include on the computers
if [ -r "$confdir/paths" ]; then
	paths=`cat $confdir/paths`
else
	echo
	echo "Paths file does not exist, exiting..."
	echo
	exit 1
fi

# make date-variables
DATE=`date "+%Y%m%d%H%M"` #format: touch
DATEs=`date "+%Y-%m-%d %H:%M"` #format: tar

# The backup
# Needs variables to be set to work
backup()
{
	echo
	echo "$computer:$remotepath starts $backuplevel backup"

	if [ "$backuplevel" = "daily" ]; then
		lastday="--newer-mtime='`cat $this_remotepath/.date`'"
	else
		lastday=""
	fi

	if (ssh -q -q $computer "tar --create --one-file-system $lastday --directory '$remotepath' . | bzip2") < /dev/null > "$this_remotepath/$backuplevel/$DATE.tmp"; then
		echo "$backuplevel backup of $computer:$remotepath was successful"
	else
		rm -f "$this_remotepath/$backuplevel/$DATE.tmp"
		echo
		echo "$backuplevel backup of $computer:$remotepath failed and deleted"
		echo
		exit 1
	fi

	if ! mv $this_remotepath/$backuplevel/$DATE.tmp $this_remotepath/$backuplevel/$DATE.tar.bz2; then
		echo
		echo "Could not save backup file, exiting..."
		echo
		exit 1
	fi

	# make a filelist.
	# update the datefile if the filelist is ok.
	if ! tar tvfj $this_remotepath/$backuplevel/$DATE.tar.bz2 > $this_remotepath/$backuplevel/$DATE.idx 2>&1 && echo $DATEs > $this_remotepath/.date && touch -t $DATE $this_remotepath/.date; then
		echo
		echo "Could not make filelist on backup of:"
		echo "$computer:$remotepath, continuing..."
		echo
	fi

	# make a sorted filelist
	if ! grep -v ^d $this_remotepath/$backuplevel/$DATE.idx | sort -n -r -k 3 > $this_remotepath/$backuplevel/$DATE.sdx; then
		echo
		echo "Could not make sorted filelist on backup of:"
		echo "$computer:$remotepath, continuing..."
		echo
	fi

	# fix permisions
	if ! chmod --quiet 600 $this_remotepath/$backuplevel/*bz2 2>/dev/null; then
		echo "Could not set permission on file $DATE.tar.bz, exiting..."
		echo
		exit 1
	fi
	if ! chmod --quiet 644 $this_remotepath/$backuplevel/*{sdx,idx} 2>/dev/null; then
		echo "Could not set permission on files $DATE.{sdx,idx}, exiting..."
		echo
		exit 1
	fi
} # end backup() function


# Go through list of computers and do backup
for computer in $computers; do
	# try to copy and run dbbackup if it exist
	if [ -f $confdir/dbbackup.$computer ]; then
		if ! ssh -q -q $computer "[ ! -d ~/.tmp ] && mkdir ~/.tmp"; then
			if ! ssh -qq $computer "[ -d ~/.tmp ]"; then
				echo
				echo "Could not create tmp directory on $computer, exiting..."
				echo
				exit 1
			fi
		fi

		if scp -B -q $confdir/dbbackup.$computer $computer:~/.tmp/dbbackup; then
			echo "Uploaded dbbackup to $computer"
		else
			echo "Could not upload dbbackup script to $computer, exiting..."
			echo
			exit 1
		fi

		if ! ssh -q -q $computer "[ ! -x ~/.tmp/dbbackup ] && chmod 700 ~/.tmp/dbbackup"; then
			if ! ssh -qq $computer "[ -x ~/.tmp/dbbackup ]"; then
				echo "Could not set mode on backup script on $computer, exiting..."
				echo
				exit 1
			fi
		fi

		if ssh -q -q $computer "[ -f ~/.tmp/dbbackup ] && /bin/sh ~/.tmp/dbbackup"; then
			echo "Run backupdb script on $computer"
		else
			echo "Could not run script on $computer, exiting..."
			echo
			exit 1
		fi
	fi

	for remotepath in $paths; do
		this_remotepath="$storagedir/$computer$remotepath"
		# make local backup directories
		if ! mkdir -m 700 -p $this_remotepath/{full,daily}; then
			echo "Could not create backup directories, exiting..."
			exit 1
		fi

		# set default backuplevel
		backuplevel="daily"

		# take the first full backup of this filesystem on this computer
		if [ ! -f $this_remotepath/.date ]; then
			backuplevel="full"
			echo $DATEs > $this_remotepath/.date
		fi

		# check if full backup is scheduled
		if [ -z "`find $this_remotepath/full/ -name \*bz2 -mtime -$days_between_full_backup`" ]; then
			backuplevel="full"
		fi

		# perform the actual backup
		backup

		# delete complete backups
		mf=$(($max_full_backup+1))
		for full in `ls -t1 $this_remotepath/full/*bz2 | tail -n +$mf`; do
			prefix=`basename $full .tar.bz2`
			if ! rm $this_remotepath/full/$prefix*; then
				echo "Could not delete old backup from $computer:$remotepath with name $prefix, exiting..."
				exit 1
			fi
		done

		# delete incremental backups older than the oldest complete backup
		oldfiles=`ls -t1 $this_remotepath/full/*bz2 | tail -1`
		if ! find $this_remotepath/daily -type f ! -newer $oldfiles -print0 | xargs -0 rm -f; then
			echo "No daily backup needed to be deleted, or failiure of deletion, continuing..."
		fi

		if [ -r $this_remotepath/$backuplevel/$DATE.tar.bz2 ];then
			echo "Backup $DATE.tar.bz2 used `du -sh $this_remotepath/$backuplevel/$DATE.tar.bz2 | gawk -F' ' '{ print $1 }'`B"
		else
			echo
			echo "Could not find backup from the remote computer. Probably script error, exiting..."
			echo
		fi
	done
done

# print diskusage to logfile
echo
echo "Local filesystem got `df -a -m | grep "$storagemount" | uniq | gawk -F' ' '{ print $4 }'` MB free space"
echo "Local filesystem is `df -a -m | grep "$storagemount" | uniq | gawk -F' ' '{ print $5 }'` used"
echo
echo "Backup script finnished at time `date`"
echo

# ===== END OF SCRIPT =====
exit 0
